@@ -11,7 +11,7 @@ from satella.coding.concurrent import TerminableThread, CallableGroup, Condition
     LockedStructure, AtomicNumber, Monitor, IDAllocator, call_in_separate_thread, Timer, \
     parallel_execute, run_as_future, sync_threadpool, IntervalTerminableThread, Future, \
     WrappingFuture, PeekableQueue, SequentialIssuer, CancellableCallback, ThreadCollection, \
-    BogusTerminableThread, SingleStartThread, FutureCollection
+    BogusTerminableThread, SingleStartThread, FutureCollection, MonitorSet
 from satella.coding.concurrent.futures import call_in_future, ExecutorWrapper
 from satella.coding.sequences import unique
 from satella.exceptions import WouldWaitMore, AlreadyAllocated, Empty
@@ -19,6 +19,12 @@ from satella.exceptions import WouldWaitMore, AlreadyAllocated, Empty
 
 class TestConcurrent(unittest.TestCase):
 
+    def test_monitor_set(self):
+        ms = MonitorSet([1, 2, 3])
+        self.assertFalse(ms.insert_and_check(2))
+        self.assertTrue(ms.insert_and_check(4))
+        self.assertFalse(ms.insert_and_check(4))
+
     def test_future_collection_exception(self):
         fc = FutureCollection([PythonFuture(), PythonFuture()])
         fc.set_running_or_notify_cancel()
@@ -136,14 +142,29 @@ class TestConcurrent(unittest.TestCase):
         tc.daemon = False
         self.assertFalse(tc.is_alive())
         self.assertEqual(len(tc), 3)
+        a = Threading(5)
+        tc += a
+        b = [Threading(8), Threading(9)]
+        tc += b
+        tc += ThreadCollection([Threading(10), Threading(11)])
         self.assertFalse(tc.daemon)
         for t in tc:
             self.assertIsInstance(t, Threading)
-        tc.append(Threading(5)).add(Threading(6))
+        tc.append(Threading(6)).add(Threading(7))
         tc.start()
         self.assertRaises(WouldWaitMore, lambda: tc.join(2))
         tc.terminate().join()
-        self.assertEqual(dct, {2: True, 3: True, 4: True, 5: True, 6: True})
+        self.assertEqual(len(tc), 10)
+        td = ThreadCollection([Threading(12), Threading(13)])
+        te = tc + td
+        self.assertEqual(len(te), 12)
+        tf = te + []
+        self.assertEqual(len(tf), 12)
+        tg = tf + Threading(15)
+        self.assertEqual(len(tg), 13)
+        self.assertEqual(dct, {2: True, 3: True, 4: True, 5: True, 6: True, 7: True, 8: True,
+                               9: True,
+                               10: True, 11: True})
 
     def test_cancellable_callback(self):
         a = {'test': True}
@@ -701,6 +722,28 @@ class TestConcurrent(unittest.TestCase):
         self.assertTrue(mtt.overrun)
         mtt.terminate().join()
 
+    def test_interval_terminable_thread_terminates(self):
+        class MyTerminableThread(IntervalTerminableThread):
+            def __init__(self, a):
+                super().__init__(1, terminate_on=ValueError)
+                self.a = a
+                self.overrun = False
+
+            def prepare(self) -> None:
+                if self.a:
+                    raise ValueError()
+
+            def loop(self) -> None:
+                if not self.a:
+                    raise ValueError()
+
+        mtt_a = MyTerminableThread(True)
+        mtt_b = MyTerminableThread(False)
+        mtt_a.start()
+        mtt_b.start()
+        mtt_a.terminate().join()
+        mtt_b.terminate().join()
+
     @unittest.skipIf(platform.python_implementation() != 'PyPy', 'this requires PyPy')
     def test_terminable_thread_force_notimplementederror(self):
         class MyTerminableThread(TerminableThread):
